# flake8: noqa: E405

"""
    Django settings for running tests
"""

from .base import *  # noqa


# _____________________________________________________________________________
# Manage Django Project

INSTALLED_APPS.append('manage_django_project.apps.ManageDjangoProjectConfig')



from unittest.mock import patch

from bx_py_utils.test_utils.snapshot import assert_text_snapshot
from cmd2.ansi import strip_style
from django.test import SimpleTestCase
from django_tools.management.commands import run_testserver
from django_tools.unittest_utils.call_management_commands import captured_call_command

from manage_django_project.management.commands import run_dev_server, shell
from manage_django_project.tests.cmd2_test_utils import BaseShellTestCase


class RunDevServerShellTestCase(BaseShellTestCase):
    def test_basic_run_dev_server(self):
        with patch.object(shell, 'verbose_check_call') as call_mock:
            stdout, stderr = self.execute(command='run_dev_server')
        self.assertEqual(stderr, '')
        self.assertIn('call command manage_django_project.run_dev_server', stdout)
        call_mock.assert_called_once()


class RunDevServerTestCase(SimpleTestCase):
    maxDiff = None

    def test_basic_run_dev_server(self):
        with patch.object(run_testserver, 'call_command') as call_command_mock:
            output, stderr = captured_call_command(
                run_dev_server,
            )

        self.assertEqual(stderr, '')

        output = strip_style(output)

        self.assertIn('makemigrations', output)
        self.assertIn('migrate', output)
        self.assertIn('runserver', output)

        call_command_mock.assert_called()

        assert_text_snapshot(got=output)



import inspect
import os
import sys
from pathlib import Path

import rich
from django.core.management import call_command
from manageprojects.test_utils.subprocess import SubprocessCallMock

from manage_django_project.tests import PROJECT_ROOT


def get_rstrip_paths():
    return (
        Path(sys.executable).parent.parent,
        PROJECT_ROOT.parent,
    )


def call_command_capture_subprocess(cmd_module) -> list[str]:
    assert inspect.ismodule(cmd_module)

    with SubprocessCallMock() as call_mock:
        call_command(cmd_module.Command())

    popenargs = call_mock.get_popenargs(rstrip_paths=get_rstrip_paths())
    return popenargs


class ForceRichTerminalWidth:
    def __init__(self, width=120):
        self.width = width
        self.origin_width = None

    def __enter__(self):
        console = rich.get_console()
        self.origin_width = console.width
        os.environ['COLUMNS'] = str(self.width)
        rich.reconfigure(width=self.width)
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        if exc_type:
            return False
        rich.reconfigure(width=self.origin_width)
        os.environ['COLUMNS'] = str(self.origin_width)



from unittest.mock import patch

from django.test import SimpleTestCase

from manage_django_project.management.commands import install, shell
from manage_django_project.tests.cmd2_test_utils import BaseShellTestCase
from manage_django_project.tests.command_test_utils import call_command_capture_subprocess


class InstallShellTestCase(BaseShellTestCase):
    def test_basic_install(self):
        with patch.object(shell, 'verbose_check_call') as call_mock:
            stdout, stderr = self.execute(command='install')
        self.assertEqual(stderr, '')
        self.assertIn('call command manage_django_project.install', stdout)
        call_mock.assert_called_once()


class InstallTestCase(SimpleTestCase):
    maxDiff = None

    def test_basic_install(self):
        popenargs = call_command_capture_subprocess(cmd_module=install)
        self.assertEqual(
            popenargs,
            [
                ['.../bin/pip', 'install', '--no-deps', '-e', '.'],
            ],
        )



from cmd2 import CommandResult
from cmd2.ansi import strip_style
from cmd2_ext_test import ExternalTestMixin
from django.test import SimpleTestCase

from manage_django_project.management.commands.shell import ManageDjangoProjectApp


class ManageDjangoTestApp(ExternalTestMixin, ManageDjangoProjectApp):
    pass


class BaseShellTestCase(SimpleTestCase):
    """
    Base class for cmd2 app test cases
    """

    def setUp(self):
        super().setUp()
        self.app = ManageDjangoTestApp()
        self.app.fixture_setup()

    def tearDown(self):
        super().tearDown()
        self.app.fixture_teardown()

    def execute(self, command, remove_colors=True, rstrip=True):
        out = self.app.app_cmd(command)

        assert isinstance(out, CommandResult)
        if out.stdout is None:
            stdout = ''
        else:
            stdout = str(out.stdout)
            if remove_colors:
                stdout = strip_style(stdout)
            if rstrip:
                stdout = '\n'.join(line.rstrip() for line in stdout.splitlines())

        if out.stderr is None:
            stderr = ''
        else:
            stderr = str(out.stderr)
            if remove_colors:
                stderr = strip_style(stderr)

        return stdout, stderr



import os
import unittest.util
from pathlib import Path

import manage_django_project


PROJECT_ROOT = Path(manage_django_project.__file__).parent.parent


# Hacky way to expand the failed test output:
unittest.util._MAX_LENGTH = os.environ.get('UNITTEST_MAX_LENGTH', 300)



import inspect
import re
from unittest.mock import patch

from bx_py_utils.test_utils.redirect import RedirectOut
from django.test import SimpleTestCase

from manage_django_project.management.commands import shell, update_req
from manage_django_project.tests.cmd2_test_utils import BaseShellTestCase
from manage_django_project.tests.command_test_utils import call_command_capture_subprocess


class UpdateReqShellTestCase(BaseShellTestCase):
    def test_basic_update_req(self):
        with patch.object(shell, 'verbose_check_call') as call_mock:
            stdout, stderr = self.execute(command='update_req')
        self.assertEqual(stderr, '')
        self.assertIn('call command manage_django_project.update_req', stdout)
        call_mock.assert_called_once()


class UpdateReqTestCase(SimpleTestCase):
    maxDiff = None

    def test_basic_update_req(self):
        with RedirectOut() as buffer:
            popenargs = call_command_capture_subprocess(cmd_module=update_req)
        self.assertEqual(
            popenargs,
            [
                ['.../bin/pip', 'install', '-U', 'pip'],
                ['.../bin/pip', 'install', '-U', 'pip-tools'],
                [
                    '.../bin/pip-compile',
                    '--allow-unsafe',
                    '--resolver=backtracking',
                    '--upgrade',
                    '--generate-hashes',
                    'pyproject.toml',
                    '--output-file',
                    'requirements.txt',
                ],
                [
                    '.../bin/pip-compile',
                    '--allow-unsafe',
                    '--resolver=backtracking',
                    '--upgrade',
                    '--generate-hashes',
                    'pyproject.toml',
                    '--extra=dev',
                    '--extra=dev',
                    '--output-file',
                    'requirements.dev.txt',
                ],
                [
                    '.../bin/pip-compile',
                    '--allow-unsafe',
                    '--resolver=backtracking',
                    '--upgrade',
                    '--generate-hashes',
                    'pyproject.toml',
                    '--extra=dev',
                    '--extra=django32',
                    '--output-file',
                    'requirements.django32.txt',
                ],
                [
                    '.../bin/pip-compile',
                    '--allow-unsafe',
                    '--resolver=backtracking',
                    '--upgrade',
                    '--generate-hashes',
                    'pyproject.toml',
                    '--extra=dev',
                    '--extra=django41',
                    '--output-file',
                    'requirements.django41.txt',
                ],
                [
                    '.../bin/pip-compile',
                    '--allow-unsafe',
                    '--resolver=backtracking',
                    '--upgrade',
                    '--generate-hashes',
                    'pyproject.toml',
                    '--extra=dev',
                    '--extra=django42',
                    '--output-file',
                    'requirements.django42.txt',
                ],
                ['.../bin/pip-sync', 'requirements.django42.txt'],
            ],
        )
        self.assertEqual(buffer.stderr, '')

        blocks = re.split(r'-{10,}', buffer.stdout)
        last_block = blocks[-1]
        self.assertEqual(
            last_block.strip(),
            inspect.cleandoc(
                '''
                Generate requirement files:
                 * requirements.txt
                 * requirements.dev.txt
                 * requirements.django32.txt
                 * requirements.django41.txt
                 * requirements.django42.txt

                Install requirement from: requirements.django42.txt
                '''
            ).strip(),
        )



import shutil
from unittest import TestCase

from bx_py_utils.path import assert_is_file
from cli_base.cli_tools.subprocess_utils import verbose_check_output

from manage_django_project import __version__
from manage_django_project.tests import PROJECT_ROOT


class ManageTestCase(TestCase):
    def test_manage_py_call(self):
        assert_is_file(PROJECT_ROOT / 'manage.py')

        system_python_bin = shutil.which('python3')
        self.assertIn('python', system_python_bin)

        output = verbose_check_output(system_python_bin, 'manage.py', '--version', verbose=False, cwd=PROJECT_ROOT)
        self.assertIn('.venv/bin/manage_django_project_example --version', output)
        self.assertIn(__version__, output)  # Check Django version will not work with tox!

        output = verbose_check_output(system_python_bin, 'manage.py', '--help', verbose=False, cwd=PROJECT_ROOT)
        self.assertIn('.venv/bin/manage_django_project_example --help', output)
        self.assertIn('manage_django_project_example help <subcommand>', output)
        self.assertIn('Available subcommands:', output)
        self.assertIn('[django]', output)
        self.assertIn('[manage_django_project]', output)
        self.assertIn('project_info', output)



from importlib import import_module
from pathlib import Path

from bx_py_utils.auto_doc import assert_readme_block
from bx_py_utils.path import assert_is_file
from django.test import SimpleTestCase

from manage_django_project.management import commands
from manage_django_project.tests import PROJECT_ROOT


def get_own_command_names() -> list[str]:
    commands_path = Path(commands.__file__).parent
    command_names = [item.stem for item in commands_path.glob('*.py') if not item.stem.startswith('_')]
    return command_names


def get_own_commands():
    command_names = get_own_command_names()
    result = {}
    for command_name in command_names:
        module = import_module(f'manage_django_project.management.commands.{command_name}')
        CommandClass = module.Command
        result[command_name] = CommandClass

    return result


def assert_cli_help_in_readme(text_block: str, marker: str):
    README_PATH = PROJECT_ROOT / 'README.md'
    assert_is_file(README_PATH)

    assert_readme_block(
        readme_path=README_PATH,
        text_block=text_block,
        start_marker_line=f'[comment]: <> (✂✂✂ auto generated {marker} start ✂✂✂)',
        end_marker_line=f'[comment]: <> (✂✂✂ auto generated {marker} end ✂✂✂)',
    )


class ReadmeTestCase(SimpleTestCase):
    def test_own_commands(self):
        commands = get_own_commands()
        self.assertGreaterEqual(len(commands), 10)
        help_info = []
        for command_name, CommandClass in sorted(commands.items()):
            if help := CommandClass.help:
                help_info.append(f'* `{command_name}` - {help}')
        help_info = '\n'.join(help_info)
        text_block = f'\n{help_info}\n'
        assert_cli_help_in_readme(text_block=text_block, marker='command_info')



from unittest.mock import patch

from bx_py_utils.test_utils.snapshot import assert_text_snapshot
from django.test import SimpleTestCase
from django_tools.unittest_utils.call_management_commands import captured_call_command

from manage_django_project import __version__
from manage_django_project.management.commands import project_info, shell
from manage_django_project.tests import PROJECT_ROOT
from manage_django_project.tests.cmd2_test_utils import BaseShellTestCase
from manage_django_project.tests.command_test_utils import ForceRichTerminalWidth


class ProjectInfoShellTestCase(BaseShellTestCase):
    def test_basic_project_info(self):
        with patch.object(shell, 'verbose_check_call') as call_mock:
            stdout, stderr = self.execute(command='project_info')
        self.assertEqual(stderr, '')
        self.assertIn('call command manage_django_project.project_info', stdout)
        call_mock.assert_called_once()


class InstallTestCase(SimpleTestCase):
    maxDiff = None

    def test_basic_install(self):
        with ForceRichTerminalWidth(width=120):
            output, stderr = captured_call_command(project_info)
        self.assertEqual(stderr, '')
        self.assertIn('manage_config = ProjectInfo(', output)
        self.assertIn("distribution_name='manage_django_project',", output)

        path_str = str(PROJECT_ROOT.parent)
        self.assertIn(path_str, output)
        stdout_output = output.replace(path_str, '...')

        self.assertIn(__version__, stdout_output)
        stdout_output = stdout_output.replace(__version__, '<mocked>')

        assert_text_snapshot(got=stdout_output)



import sys
from pathlib import Path

from cli_base.cli_tools.subprocess_utils import verbose_check_call
from django_rich.management import RichCommand
from manageprojects.utilities.pyproject_toml import TomlDocument, get_pyproject_toml


class Command(RichCommand):
    help = 'Update project requirements via pip-tools'

    def handle(self, *args, **options):
        self.console.print(f'\n[bold]{self.help}')
        verbose = options['verbosity'] > 1

        toml_document: TomlDocument = get_pyproject_toml()
        self.console.print(f'Use: {toml_document.file_path}')

        pyproject_config = toml_document.doc.unwrap()  # TOMLDocument -> dict

        project_cfg = pyproject_config['project']
        assert 'dependencies' in project_cfg, 'No "dependencies" in [project] in "pyproject.toml" found!'

        bin_path = Path(sys.executable).parent

        verbose_check_call(bin_path / 'pip', 'install', '-U', 'pip')
        verbose_check_call(bin_path / 'pip', 'install', '-U', 'pip-tools')

        extra_env = dict(
            CUSTOM_COMPILE_COMMAND='./manage.py update_req',
        )

        pip_compile_base = [
            bin_path / 'pip-compile',
            '--allow-unsafe',  # https://pip-tools.readthedocs.io/en/latest/#deprecations
            '--resolver=backtracking',  # https://pip-tools.readthedocs.io/en/latest/#deprecations
            '--upgrade',
            '--generate-hashes',
        ]
        if verbose:
            pip_compile_base.append('--verbose')

        # Only "prod" dependencies:
        verbose_check_call(
            *pip_compile_base,
            'pyproject.toml',
            '--output-file',
            'requirements.txt',
            extra_env=extra_env,
        )
        requirements_names = ['requirements.txt']
        last_requirements_name = 'requirements.txt'

        if opt_deps := pyproject_config['project'].get('optional-dependencies'):
            opt_dep_names = opt_deps.keys()
            if opt_dep_names:
                assert 'dev' in opt_dep_names, 'No "dev" list in [project.optional-dependencies] found!'
                for name in opt_dep_names:
                    requirements_name = f'requirements.{name}.txt'
                    verbose_check_call(
                        *pip_compile_base,
                        'pyproject.toml',
                        '--extra=dev',
                        f'--extra={name}',
                        '--output-file',
                        requirements_name,
                        extra_env=extra_env,
                    )
                    last_requirements_name = requirements_name
                    requirements_names.append(requirements_name)

        # Install new dependencies in current .venv:
        verbose_check_call('pip-sync', last_requirements_name)

        self.console.print('\n')
        self.console.print('-' * 100)
        self.console.print('[green]Generate requirement files:')
        for requirements_name in requirements_names:
            self.console.print(f'[bold] * [cyan]{requirements_name}')

        self.console.print(f'\n[green]Install requirement from: [bold]{last_requirements_name}\n')
